@@ -34,6 +34,7 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 
 #define TEST_ASSERT(expression, message) \
 	G_STMT_START { if (!(expression)) test_failed message; } G_STMT_END
@@ -92,7 +93,7 @@ test_make_canonical_path (const char *input,
 	g_free (output);
 }
 
-const char* test_uris[][2] = 
+static const char* test_uris[][2] = 
 {
 	{ "http://www.gnome.org/", "index.html" },
 	{ "http://www.gnome.org/", "/index.html"},
